<?php

if (!defined('_CONFIGEXPORTIMPORTTOOL_CLASS')) {
	define('_CONFIGEXPORTIMPORTTOOL_CLASS', true);

	include('/usr/php/include/verySimpleXmlParser.php');
	include_once("/usr/www/include/SQLite2ToSQLite3.php");
	include_once '/usr/fallback/beroConf.php';
	require_once('/usr/local/www/berogui/includes/Helper/Helper.php');

	class configExportImportTool {

		private $_configPath;
		private $_xmlData;

		private $_ba;
		private $_is_sbc_vm = false;
		/**
		 * is dynamically populated based on whetever a config file is given when object is created, based on the file provided, or all files for export
		 */
		private $_fileList;
		private $_fileData;
		private $_fileGroups;

		/**
		 * if a file with appConfigs is present, set to true, else set to false
		 */
		private $_appConfig=false;
		private $_provisionConfigUpdate = false;
		private $_provisionActivate = false;
		private $_plainPassword;

		
		function __construct ($fileName = null) {
			if (!file_exists('/usr/local/www/berogui/misc/database.php')) {
				throw new Exception('missing database script'); 
			}
			include_once("/usr/local/www/berogui/misc/database.php");
			$this->_ba = new beroAri();
			$ba = $this->_ba;
			require_once("/usr/local/www/berogui/includes/lang.php");

			$this->_is_sbc_vm = $this->_ba->get('is_sbc_vm');
			
			if ($this->_is_sbc_vm) {
				$this->_configPath = "/etc/beroNet/isgw/";
				$this->_fileList = array('isgw.conf', 'isgw.dialplan', 'isgw.sip', 'isgw.whitelist', 'misc.conf', 'network.acl');
			}
			else {
				$this->_configPath  = "/usr/conf/";
				$this->_fileList = array('hardware.conf', 'isgw.causes', 'isgw.conf', 'isgw.cpt', 'isgw.dialplan', 
										 'isgw.sip', 'isgw.whitelist', 'misc.conf', 'network.acl');
				$this->_fileList = array_merge($this->_fileList,$this->_setFilesOfModulesInstalled());
			}

			$this->_xmlData = null;
			// file reading if file is present
			if ($fileName != null) {
				if (substr($fileName, -3) == '.gz') {
					exec("gzip -d $fileName");
					$fileName = substr($fileName, 0, -3);
				}
				if (($this->_xmlData = xmlParseData(file_get_contents($fileName))) === false) {
					exit(false);
				}
				$this->_fileList = array();
				$this->_setConfigData();
				$filesNotPresent =array_diff(array('isgw.analog',"isgw.gsm","isgw.isdn","isgw.cas","isgw.lte"),$this->_setFilesOfModulesInstalled());
				$this->_fileList = array_diff($this->_fileList,$filesNotPresent);
			}
			$this->_setFileGroups();

		}

		/**
		 * Checks module specified with @param $lifToCheck, for whetever the cas mode is activated for that module
		 *
		 * @param int index of lif to check for cas mode
		 * @return bool true if cas mode is active, else false
		 */
		private function _checkCasMode($lifToCheck){
			if (file_exists('/usr/conf/hardware.conf')){ 
				$hardwareConf =@file_get_contents('/usr/conf/hardware.conf');
				$lifConf =explode('[lif',$hardwareConf);
				if(strpos($lifConf[$lifToCheck+1],'interface_type=cas') !==false) return true;
				else return false;
			}
			else return false;
		}

		/**
		 * gets modules of gateway and returns the corresponding configuration files
		 * @return array
		 */
		private function _setFilesOfModulesInstalled(){
			$modulefiles = array();
			foreach(rootHelper::getLif() as $lifIndex => $module){
				$module = ltrim($module, "bf");
				switch ($module) {
					case '2S02FXS':
						if(!in_array("isgw.analog",$modulefiles,1))array_push($modulefiles, "isgw.analog");
						if(!in_array("isgw.gsm",$modulefiles,1))array_push($modulefiles, "isgw.gsm");
						break;
					
					case preg_match('/[1248]FX[O]/', $module)==1:
						if(!in_array("isgw.analog",$modulefiles,1))array_push($modulefiles, "isgw.analog");
						break;
					
					case preg_match('/[124]S0|[12]E1/',$module)==1:
						if(!in_array("isgw.isdn",$modulefiles,1))array_push($modulefiles, "isgw.isdn");
						break;
					case preg_match('/[12]LTE/', $module)==1:
						if(!in_array("isgw.lte",$modulefiles,1))array_push($modulefiles, "isgw.lte");
						break;
					case  preg_match('/[124]t1e1/', $module)==1:
						if($this->_checkCasMode($lifIndex)==true &&!in_array("isgw.cas",$modulefiles,1))array_push($modulefiles, "isgw.cas");
						else if(!in_array("isgw.isdn",$modulefiles,1))array_push($modulefiles, "isgw.isdn");
						break;
					case preg_match('/[12]GSM/', $module)==1:
						if(!in_array("isgw.gsm",$modulefiles,1))array_push($modulefiles, "isgw.gsm");

					default:
						break;
				}
			}
			return $modulefiles;
		}
		
		/**
		 * set all data from the <code><config></code> section of the provided file
		 */
		private function _setConfigData(){
			if ($this->_getSectionByName('AppConfig')!==false) $this->_appConfig =true; 
			if (($config = $this->_getSectionByName('Config')) === false) {
				return (false);
			}

			foreach ($config as $cfg_section) {
				if (!is_array($cfg_section)) continue;
				$i = 0;

				foreach ($cfg_section as $cfg_file) {
					//set config options
					if ($cfg_file['name'] == 'PlainPassword' && ($cfg_file['data'] ==='yes'|| $cfg_file['data'] ==='true')) {
						$this->_plainPassword =true;
						continue;
					}
					if ($cfg_file['name'] == 'Update') {
						if($cfg_file['data'] == 'yes') $this->_provisionConfigUpdate = true;
						continue;
					}
					if ($cfg_file['name'] == 'Activate') {
						$this->_provisionActivate = $cfg_file['data'];
					}
					
					//set fileList and fileData
					if($cfg_file['name']=='File'){
						foreach($cfg_file['subsection'] as $index => $file){
							switch($file['name']) {
								case 'Name':
									array_push($this->_fileList, $file['data']);
									$this->_fileData[$i]['name'] = $file['data'];	
									break;

								case 'Contents':	
									$this->_fileData[$i]['data'] = $file['data'];	
									break;
							}
						}
						$i++;
					}
				}
			}
		}

		
		private function _exportGatewayConfig ($confFiles) {
			unset($files);
			foreach ($confFiles as $confFile) {
				$files .=	"\t\t" . '<File>' . "\n" .
						"\t\t\t" . '<Name>' . $confFile . '</Name>' . "\n" .
						"\t\t\t" . '<Contents>' . "\n" .
						file_get_contents($this->_configPath . '/' . $confFile) . "\n" .
						"\t\t\t" . '</Contents>' . "\n" .
						"\t\t" . '</File>' . "\n";
			}

			$ret =	"\t" . '<Config>' . "\n" .
				"\t\t" . '<PlainPassword>false</PlainPassword>' . "\n" .
				"\t\t" . '<Update>yes</Update>' . "\n" .
				"\t\t" . '<Activate>3</Activate>' . "\n" .
				$files .
				"\t" . '</Config>' . "\n";

			return($ret);

		}

		private function _isBinary ($fileName, $maxLimit = 1024) {

			$size = filesize($fileName);
			$limit = ($size > $maxLimit) ? $maxLimit : $size;
			$data = file_get_contents($fileName, false, null, -1, $limit);

			// everything with char \0 is binary
			for ($i = 0; $i < $limit; $i++) {
				if ($data[$i] == "\0") {
					return(true);
				}
			}

			// XML-files are also handled as binary data
			if (strpos($data, '<?xml') !== false) {
				return(true);
			}

			return(false);
		}

		private function _getFileList ($dirName, &$dirArray) {

			$files = array_diff(scandir($dirName), array('.', '..'));
			if (empty($files)) {
				return;
			}

			foreach($files as $val) {
				$dirArray[$dirName][] = $val;
				if(is_dir($dirName . '/' . $val)) {
					$this->_getFileList($dirName . '/' . $val, $dirArray);
				}
			}
			@ksort($dirArray);
		}

		private function _getAppConfig () {

			$baseDir = '/usr/conf/userapp';

			if (!is_dir($baseDir)) {
				return(null);
			}

			$apps = array_diff(scandir($baseDir), array('.', '..'));

			$i = 0;
			foreach ($apps as $app) {

				// get config-files of app
				unset($appFiles);
				$this->_getFileList($baseDir . '/' . $app, $appFiles);
				if (empty($appFiles)) {
					continue;
				}

				// save all info in array
				$ret[$i]['name'] = $app;
				$j = 0;
				foreach ($appFiles as $key => $val) {

					foreach ($val as $item) {
						$filePath = $key . '/' . $item;
						$fileMode = $this->_isBinary($filePath);
						$ret[$i]['files'][$j]['name'] = strstr($filePath, $app);
						if (is_dir($filePath)) {
							$ret[$i]['files'][$j]['mode'] = 'directory';
							$j++;
							continue;
						}
						$ret[$i]['files'][$j]['mode'] = ($fileMode ? 'binary' : 'text');
						$contentRaw = file_get_contents($filePath);
						$ret[$i]['files'][$j]['content'] = ($fileMode ? base64_encode($contentRaw) : trim($contentRaw));
						unset($contentRaw);
						$j++;
					}
				}
				$i++;
			}

			return(empty($ret) ? null : $ret);
		}

		private function _exportAppConfig () {

			if (($apps = $this->_getAppConfig()) == null) {
				return('');
			}

			$ret =	"\t" . '<AppConfig>' . "\n";

			foreach ($apps as $app) {
				if (empty($app['files'])) {
					continue;
				}

				$ret .=	"\t\t" . '<App>' . "\n" .
					"\t\t\t" . '<Name>' . $app['name'] . '</Name>' . "\n";

				foreach ($app['files'] as $appFile) {
					$ret .= "\t\t\t" . '<File>' . "\n" .
						"\t\t\t\t" . '<Name>' . $appFile['name'] . '</Name>' . "\n" .
						"\t\t\t\t" . '<Mode>' . $appFile['mode'] . '</Mode>' . "\n" .
						(($appFile['mode'] == 'directory') ? '' :	"\t\t\t\t" . '<Content>' . "\n" .
												$appFile['content'] . "\n" .
												"\t\t\t\t" . '</Content>' . "\n") .
						"\t\t\t" . '</File>' . "\n";
				}
				$ret .= "\t\t" . '</App>' . "\n";
			}

			$ret .=	"\t" . '</AppConfig>' . "\n";

			return($ret);
		}

		private function _getSectionByName ($name) {

			if ($this->_xmlData == null) {
				return(false);
			}

			// check if this file is valid
			if ($this->_xmlData['name'] != 'beroNetProvisioning') {
				return(false);
			}

			// get the section with config-files
			foreach ($this->_xmlData['subsection'] as $section) {
				if ($section['name'] == $name) {
					return($section);
				}
			}

			return(false);
		}

		/**
		 *
		 * @return bool true if appconfiguration present in provided file else false
		 */
		public function getAppInfo(){
			return $this->_appConfig;
		}

		/**
		 * sets the file groups for the GUI based on files and modules present in the config and gateway
		 */
		private function _setFileGroups(){
			$ba = new beroAri();

			$this->_fileGroups = array(
				BACKUP_RESTORE_CONFIG_GROUP_MISC_ISGW		=>array('isgw.causes','isgw.cpt'),
				BACKUP_RESTORE_CONFIG_GROUP_MISC			=>array('isgw.conf','misc.conf'),
				BACKUP_RESTORE_CONFIG_GROUP_HARDWARE		=> 'hardware.conf',
				BACKUP_RESTORE_CONFIG_GROUP_DIALPLAN		=> 'isgw.dialplan',
				BACKUP_RESTORE_CONFIG_GROUP_VOIP			=>'isgw.sip',
				BACKUP_RESTORE_CONFIG_GROUP_ACL				=>array('isgw.whitelist','network.acl'),
			);
			//remove groups that are not in imported file
			foreach ($this->_fileGroups as $group => $files) {
				if (is_array($files)) {
					foreach ($files as  $key =>$file) {
						if(!in_array($file,$this->_fileList)) unset($files[$key]);
					}
					if(!$files) unset($this->_fileGroups[$group]);
				} else {
					if(!in_array($files,$this->_fileList)) unset($this->_fileGroups[$group]);
				}
			}
			//add module configs; remove <isgw.> part of filename and add to _fileGroups
			foreach ($this->_fileList as $key => $fileName) {
				if(preg_match('/isgw\.(analog|isdn|gsm|cas|lte)/',$fileName) ==1) {
					$isgwGroupName =str_replace('isgw.','',$fileName.' '.BACKUP_RESTORE_CONFIG_GROUP_SETTINGS);
					$this->_fileGroups =array_merge($this->_fileGroups,array( $isgwGroupName=>$fileName));
				}
			}
		}

		/**
		 * converts the filegroups to the corresponding files
		 * @param array  filegroups to convert
		 */
		public function fileGroupsToFiles($groups){
			$filesToImport = array();
			if(is_array($groups)) $filelist =array_intersect_key($this->_fileGroups, $groups);
			else $filelist = array_intersect_key($this->_fileGroups, array($groups));
			foreach ($filelist as $files) {
				if (is_array($files)){
					foreach ($files as $file) {
						array_push($filesToImport,$file);
					}
				}
				else array_push($filesToImport,$files);
			}
			return $filesToImport;
		}

		/**
		 * generates a configfile with current device settings
		 * @param array the configfiles to be included in backup. If not set, then all configfiles will be exported 
		 * @param bool if false appconfigurations will not be exported, by default apps will be exported
		 * @return string the configfile
		 */
		public function exportConfig ($configs=NULL,$appConfig=true , $groups =false) {
			if($groups ==true) $configs = $this->fileGroupsToFiles($configs);
			if ($configs === NULL) $configs = $this->_fileList;
			$ret = '<?xml version="1.0"?>' . "\n" .
				'<beroNetProvisioning>' . "\n" .
				"\t" . '<!-- Created on ' . date('c') . ' -->' . "\n";
				$ret.= $this->_exportGatewayConfig($configs);
			if($appConfig==true)
				$ret .= $this->_exportAppConfig();
			$ret .= '</beroNetProvisioning>' . "\n";

			return($ret);
		}

		/**
		 *  imports configfiles specified with param <code>$configfiles</code> and encrypts password if set
		 * @param  array $name Array of the configfiles to be imported
		 * @return array|bool on success: array with information about success=>['info'] and failure=>['warning'] of the file import
		 */
		##TODO use the function for the cloud sbc && make getconfigfiles private
		public function importConfigFiles($configFiles=null){
			$configFiles= $this->getConfigFiles($configFiles);
			if ($configFiles ===false) return(false);
			// create backup-directory
			if (!is_dir($this->_configPath . 'backup')) {
				rootHelper::mkdir(0, "{$this->_configPath}/backup");
			}
			// go over each configFile
			foreach ($configFiles as $key=> $configFile) {
				//if Plain Password set to true, then encrypt that password
				if ($configFile['name']=='misc.conf' &&$this->_plainPassword==true){
					include_once('/usr/local/www/berogui/includes/password_func.php');
					if (preg_match('/admin-pass=(.+)/',$configFile['data'],$encode)==1){
						$encode = tep_encrypt_password($encode[1]);
						$configFile['data']=preg_replace("/admin-pass=.*/",'admin-pass='.$encode,$configFile['data']);
					}
				}
				// create backup of file
				rootHelper::copy(0, "{$this->_configPath}/{$configFile['name']}", "{$this->_configPath}/backup/{$configFile['name']}");
				rootHelper::unlink(0, "{$this->_configPath}/{$configFile['name']}");

				// write file
				if (!($FP = fopen("{$this->_configPath}/{$configFile['name']}", 'w'))) {
					$log['warning'][$key]='could_not_write'. $configFile['name'];
					rootHelper::copy(0, "{$this->_configPath}/backup/{$configFile['name']}", "{$this->_configPath}/{$configFile['name']}");
					continue;
				}

				fwrite($FP, $configFile['data'], strlen($configFile['data']));
				fclose($FP);
				$FP = null;
				$log['info'][$key]=('updated'.$configFile['name']);

			}
			return $log;
		}

		/**
		 * returns data of configFiles specified with <code>configsToGet</code>
		 * @param array list with names of files to get. if not set, all files will be returned
		 * @return array|bool
		 */
		##TODO can be made private and auto used when the cloud SBC uses the import function instead of writing the files itself
		public function getConfigFiles ($configsToGet =null) {
			$return = array();
			if($configsToGet ===null) return((count($this->_fileData) > 0) ? $this->_fileData : false);
			else{
				foreach( $this->_fileData as $file){
					if(in_array($file['name'],$configsToGet)) array_push($return,$file);
				}
			}
			return((count($return) > 0) ? $return : false);
		}

		public function getConfigActivate () {
			return $this->_provisionActivate;
		}

		/**
		 * returns true if <code><update></code> in the config is set to yes, else return false
		 */
		public function getConfigEnabled ($section) {
			return $this->_provisionConfigUpdate;
		}

		public function getFileGroups(){
			return $this->_fileGroups;
		}

		public function getAppConfigFiles () {

			if ($this->_xmlData == null) {
				return(false);
			}

			if (($apps = $this->_getSectionByName('AppConfig')) === false) {
				return(false);
			}

			foreach ($apps as $app) {
				if (!is_array($app)) {
					continue;
				}

				$i = 0;
				foreach ($app as $appConfig) {
					$j = 0;
					foreach ($appConfig['subsection'] as $appConfigSub) {
						if ($appConfigSub['name'] == 'Name') {
							$ret[$i]['name'] = $appConfigSub['data'];
						}

						if (!is_array($appConfigSub['subsection'])) {
							continue;
						}

						foreach ($appConfigSub['subsection'] as $confFileAttr) {
							switch ($confFileAttr['name']) {
							case 'Name':	$ret[$i]['files'][$j]['name'] = $confFileAttr['data'];		break;
							case 'Mode':	$ret[$i]['files'][$j]['mode'] = $confFileAttr['data'];		break;
							case 'Content':	$ret[$i]['files'][$j]['content'] = $confFileAttr['data'];	break;
							}
						}
						$j++;
					}

					$i++;
				}
			}

			return((count($ret) > 0) ? $ret : false);
		}

		public function getFirmwareInstallInfo () {

			if ($this->_xmlData == null) {
				return(false);
			}

			if (($attributes = $this->_getSectionByName('Firmware')) === false) {
				return(false);
			}

			unset($ret);
			foreach ($attributes['subsection'] as $attribute) {
				switch ($attribute['name']) {
				case 'Install':	 $ret['install'] = $attribute['data'];	break;
				case 'Filename': $ret['package'] = $attribute['data'];	break;
				case 'Url':      $ret['url'] = $attribute['data']; break;
				}
			}

			return(isset($ret) ? $ret : false);
		}

		public function getAppInstallInfo () {

			if ($this->_xmlData == null) {
				return(false);
			}

			if (($apps = $this->_getSectionByName('Apps')) === false) {
				return(false);
			}

			unset($ret);
			$i = 0;
			foreach ($apps['subsection'] as $app) {
				if (!is_array($app)) {
					continue;
				}

				foreach ($app['subsection'] as $app_attr) {
					switch ($app_attr['name']) {
					case 'Install':		$ret[$i]['install'] = $app_attr['data'];	break;
					case 'Filename':	$ret[$i]['package'] = $app_attr['data'];	break;
					}
				}
				$i++;
			}

			return($ret);
		}

	}

} /* _CONFIGEXPORTIMPORTTOOL_CLASS */

?>
